@@ -29,7 +29,6 @@
 import os
 import tempfile
 
-from debian.changelog import Version
 from debmutate.changelog import ChangelogEditor, upstream_merge_changelog_line
 from debmutate.versions import new_package_version
 
@@ -37,15 +36,11 @@ from ... import osutils
 from ...revision import NULL_REVISION
 from ...trace import note
 
-from .errors import (
-    BzrError,
-    )
 from .import_dsc import (
     DistributionBranch,
     DistributionBranchSet,
-    PreviousVersionTagMissing,
     UpstreamAlreadyImported,
-    )
+)
 from .repack_tarball import repack_tarball
 from .util import (
     component_from_orig_tarball,
@@ -53,7 +48,7 @@ from .util import (
     FORMAT_3_0_QUILT,
     FORMAT_3_0_NATIVE,
     tree_get_source_format,
-    )
+)
 
 
 TAG_PREFIX = "upstream-"
@@ -61,7 +56,7 @@ TAG_PREFIX = "upstream-"
 
 # TODO(jelmer): Move into debmutate
 def changelog_add_new_version(
-        tree, subpath, upstream_version, distribution_name, changelog,
+        tree, subpath, upstream_version: str, distribution_name, changelog,
         package):
     """Add an entry to the changelog for a new version.
 
@@ -113,7 +108,7 @@ def do_import(
       committer: Committer string to use
       files_excluded: Files to exclude
     Returns:
-      list with (component, tag, revid, pristine_tar_imported)
+      list with (component, tag, revid, pristine_tar_imported, subpath)
       tuples
     """
     db = DistributionBranch(tree.branch, tree.branch, tree=tree)
@@ -129,7 +124,8 @@ def do_import(
                 package, current_version, tempdir)
         else:
             db.create_empty_upstream_tree(tempdir)
-        if db.pristine_upstream_source.has_version(package, version, try_hard=False):
+        if db.pristine_upstream_source.has_version(
+                package, version, try_hard=False):
             raise UpstreamAlreadyImported(version)
 
         parents = {None: []}
@@ -146,7 +142,7 @@ def do_import(
 
 
 def do_merge(
-        tree, subpath, tarball_filenames, package, version,
+        tree, tarball_filenames, package, version,
         current_version, upstream_branch, upstream_revisions, merge_type=None,
         force=False, force_pristine_tar=False, committer=None,
         files_excluded=None):
@@ -246,12 +242,12 @@ def get_upstream_branch_location(tree, subpath, config, trust_package=False):
 def get_existing_imported_upstream_revids(
         upstream_source, package, new_upstream_version):
     imported_revids = []
-    for component, revid in upstream_source.version_as_revisions(
-        package, new_upstream_version
-    ).items():
+    for (component,
+         (revid, subpath)) in upstream_source.version_as_revisions(
+             package, new_upstream_version).items():
         upstream_tag = upstream_source.tag_name(
             new_upstream_version, component
         )
         imported_revids.append(
-            (component, upstream_tag, revid, None))
+            (component, upstream_tag, revid, None, subpath))
     return imported_revids
