@@ -22,21 +22,21 @@ from debian.changelog import Changelog
 import os
 import shutil
 import subprocess
-import sys
 import tempfile
 
 from ... import errors as bzr_errors
 from ...trace import note
 
+from ...transport import NoSuchFile, FileExists
+
 from .util import (
     export_with_nested,
     extract_orig_tarballs,
     get_parent_dir,
-    recursive_copy,
     )
 
 
-class SourceDistiller(object):
+class SourceDistiller:
     """A source distiller extracts the source to build from a location.
 
     It does whatever is needed to give you a source you can build at
@@ -73,7 +73,7 @@ class NativeSourceDistiller(SourceDistiller):
         :param tree: The tree to use as the source.
         :param subpath: subpath in the tree where the package lives
         """
-        super(NativeSourceDistiller, self).__init__(tree, subpath)
+        super().__init__(tree, subpath)
         self.use_existing = use_existing
 
     def distill(self, target):
@@ -87,7 +87,7 @@ class NativeSourceDistiller(SourceDistiller):
         """
         if not self.use_existing:
             if os.path.exists(target):
-                raise bzr_errors.FileExists(target)
+                raise FileExists(target)
         export_with_nested(self.tree, target, subdir=self.subpath)
 
 
@@ -102,7 +102,7 @@ class FullSourceDistiller(SourceDistiller):
         :param upstream_provider: an UpstreamProvider to provide the upstream
             tarball if needed.
         """
-        super(FullSourceDistiller, self).__init__(tree, subpath)
+        super().__init__(tree, subpath)
         self.upstream_provider = upstream_provider
         self.use_existing = use_existing
 
@@ -117,7 +117,7 @@ class FullSourceDistiller(SourceDistiller):
         """
         if not self.use_existing:
             if os.path.exists(target):
-                raise bzr_errors.FileExists(target)
+                raise FileExists(target)
         parent_dir = get_parent_dir(target)
         self.upstream_provider.provide(parent_dir)
         export_with_nested(self.tree, target, subdir=self.subpath)
@@ -139,7 +139,7 @@ class MergeModeDistiller(SourceDistiller):
         :param use_existing: whether the distiller should re-use an existing
             target if the distiller supports it.
         """
-        super(MergeModeDistiller, self).__init__(tree, subpath)
+        super().__init__(tree, subpath)
         self.upstream_provider = upstream_provider
         self.top_level = top_level
         self.use_existing = use_existing
@@ -155,7 +155,7 @@ class MergeModeDistiller(SourceDistiller):
         """
         if not self.use_existing:
             if os.path.exists(target):
-                raise bzr_errors.FileExists(target)
+                raise FileExists(target)
         elif self.use_existing:
             if not os.path.exists(target):
                 raise bzr_errors.NotADirectory(target)
@@ -183,11 +183,8 @@ class MergeModeDistiller(SourceDistiller):
             # use_existing
             if os.path.exists(os.path.join(target, 'debian')):
                 shutil.rmtree(os.path.join(target, 'debian'))
-            if sys.version_info >= (3, 8):
-                shutil.copytree(
-                    tempdir, target, symlinks=True, dirs_exist_ok=True)
-            else:
-                recursive_copy(tempdir, target)
+            shutil.copytree(
+                tempdir, target, symlinks=True, dirs_exist_ok=True)
 
 
 class DebcargoError(bzr_errors.BzrError):
@@ -216,7 +213,7 @@ class DebcargoDistiller(SourceDistiller):
         :param tree: The tree to use as the source.
         :param subpath: subpath in the tree where the package lives
         """
-        super(DebcargoDistiller, self).__init__(tree, subpath)
+        super().__init__(tree, subpath)
         self.top_level = top_level
         self.use_existing = use_existing
 
@@ -231,7 +228,7 @@ class DebcargoDistiller(SourceDistiller):
         """
         from debmutate.debcargo import parse_debcargo_source_name
         if os.path.exists(target):
-            raise bzr_errors.FileExists(target)
+            raise FileExists(target)
         with self.tree.get_file(os.path.join(
                 self.subpath,
                 'debian/changelog' if not self.top_level else
@@ -250,7 +247,7 @@ class DebcargoDistiller(SourceDistiller):
         try:
             debcargo_text = self.tree.get_file_text(
                 os.path.join(*debcargo_path))
-        except bzr_errors.NoSuchFile:
+        except NoSuchFile:
             semver_suffix = False
         else:
             from toml.decoder import loads as loads_toml
@@ -273,5 +270,5 @@ class DebcargoDistiller(SourceDistiller):
                 '--config', self.tree.abspath(os.path.join(*debcargo_path)),
                 '--directory', target, crate]
                 + ([crate_version] if crate_version else []))
-        except subprocess.CalledProcessError:
-            raise DebcargoError()
+        except subprocess.CalledProcessError as e:
+            raise DebcargoError() from e
