@@ -84,13 +84,14 @@ class VcsHgUrlToBzrUrlTests(TestCase):
     def test_with_branch(self):
         self.assertEqual(
             'https://bitbucket.org/jelmer/dulwich,branch=foo',
-            vcs_hg_url_to_bzr_url('https://bitbucket.org/jelmer/dulwich -b foo'))
+            vcs_hg_url_to_bzr_url(
+                'https://bitbucket.org/jelmer/dulwich -b foo'))
 
 
 class VcsCvsUrlToBzrUrlTests(TestCase):
 
     def setUp(self):
-        super(VcsCvsUrlToBzrUrlTests, self).setUp()
+        super().setUp()
         import breezy
         if breezy.version_info < (3, 1, 1):
             self.skipTest('version of breezy too old')
