@@ -26,7 +26,6 @@ from base64 import (
     )
 import configparser
 from debian.copyright import globs_to_re
-from debian.changelog import Version
 import errno
 from io import BytesIO
 import os
@@ -50,21 +49,22 @@ from .... import (
     osutils,
     revision as _mod_revision,
     )
+from ....branch import Branch
 from ....commit import NullCommitReporter
 from ....errors import (
     BzrError,
     DivergedBranches,
     NoSuchRevision,
     NoSuchTag,
-    NoSuchFile,
     NotBranchError,
     )
-from ....revision import NULL_REVISION
+from ....revision import NULL_REVISION, RevisionID
 from ....trace import (
     mutter,
     note,
     warning,
     )
+from ....transport import NoSuchFile
 
 from .branch import (
     git_snapshot_data_from_version,
@@ -146,7 +146,7 @@ def reconstruct_pristine_tar(dest, delta, dest_filename):
                 stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     except OSError as e:
         if e.errno == errno.ENOENT:
-            raise PristineTarError("pristine-tar is not installed")
+            raise PristineTarError("pristine-tar is not installed") from e
         else:
             raise
     (stdout, stderr) = proc.communicate(delta)
@@ -174,7 +174,7 @@ def commit_pristine_tar(dest, tarball_path, upstream=None, committer=None):
                 stderr=subprocess.PIPE, env=env)
     except OSError as e:
         if e.errno == errno.ENOENT:
-            raise PristineTarError("pristine-tar is not installed")
+            raise PristineTarError("pristine-tar is not installed") from e
         else:
             raise
     (stdout, stderr) = proc.communicate()
@@ -182,7 +182,7 @@ def commit_pristine_tar(dest, tarball_path, upstream=None, committer=None):
         if b'excessively large binary delta' in stderr:
             raise PristineTarDeltaTooLarge(stderr)
         elif b'No space left on device' in stderr:
-            raise IOError(
+            raise OSError(
                 errno.ENOSPC,
                 'Generating pristine tar delta failed: '
                 'no space left on device')
@@ -211,7 +211,7 @@ def make_pristine_tar_delta(dest, tarball_path):
                 stderr=subprocess.PIPE)
     except OSError as e:
         if e.errno == errno.ENOENT:
-            raise PristineTarError("pristine-tar is not installed")
+            raise PristineTarError("pristine-tar is not installed") from e
         else:
             raise
     (stdout, stderr) = proc.communicate()
@@ -238,7 +238,7 @@ def make_pristine_tar_delta_from_tree(
             if 'pristine-tar' in debug.debug_flags:
                 revno, revid = tree.branch.last_revision_info()
                 preserved = osutils.pathjoin(osutils.dirname(tarball_path),
-                                             'orig-%s' % (revno,))
+                                             'orig-{}'.format(revno))
                 mutter('pristine-tar failed for delta between %s rev: %s'
                        ' and tarball %s'
                        % (tree.basedir, (revno, revid), tarball_path))
@@ -252,18 +252,18 @@ def make_pristine_tar_delta_from_tree(
 
 
 def revision_has_pristine_tar_delta(rev):
-    return (u'deb-pristine-delta' in rev.properties
-            or u'deb-pristine-delta-bz2' in rev.properties
-            or u'deb-pristine-delta-xz' in rev.properties)
+    return ('deb-pristine-delta' in rev.properties
+            or 'deb-pristine-delta-bz2' in rev.properties
+            or 'deb-pristine-delta-xz' in rev.properties)
 
 
 def revision_pristine_tar_delta(rev):
-    if u'deb-pristine-delta' in rev.properties:
-        uuencoded = rev.properties[u'deb-pristine-delta']
-    elif u'deb-pristine-delta-bz2' in rev.properties:
-        uuencoded = rev.properties[u'deb-pristine-delta-bz2']
-    elif u'deb-pristine-delta-xz' in rev.properties:
-        uuencoded = rev.properties[u'deb-pristine-delta-xz']
+    if 'deb-pristine-delta' in rev.properties:
+        uuencoded = rev.properties['deb-pristine-delta']
+    elif 'deb-pristine-delta-bz2' in rev.properties:
+        uuencoded = rev.properties['deb-pristine-delta-bz2']
+    elif 'deb-pristine-delta-xz' in rev.properties:
+        uuencoded = rev.properties['deb-pristine-delta-xz']
     else:
         assert revision_has_pristine_tar_delta(rev)
         raise AssertionError(
@@ -272,11 +272,11 @@ def revision_pristine_tar_delta(rev):
 
 
 def revision_pristine_tar_format(rev):
-    if u'deb-pristine-delta' in rev.properties:
+    if 'deb-pristine-delta' in rev.properties:
         return 'gz'
-    elif u'deb-pristine-delta-bz2' in rev.properties:
+    elif 'deb-pristine-delta-bz2' in rev.properties:
         return 'bz2'
-    elif u'deb-pristine-delta-xz' in rev.properties:
+    elif 'deb-pristine-delta-xz' in rev.properties:
         return 'xz'
     assert revision_has_pristine_tar_delta(rev)
     raise AssertionError(
@@ -285,9 +285,18 @@ def revision_pristine_tar_format(rev):
 
 class BasePristineTarSource(UpstreamSource):
 
+    branch: Branch
+
     def _has_revision(self, revid, md5):
         raise NotImplementedError(self._has_revision)
 
+    def possible_tag_names(self, package: Optional[str], version: str,
+                           component: Optional[str], try_hard: bool = True):
+        raise NotImplementedError(self.possible_tag_names)
+
+    def tag_name(self, version, component=None, distro=None):
+        raise NotImplementedError(self.tag_name)
+
     def tag_version(self, version, revid, component=None):
         """Tags the upstream branch's last revision with an upstream version.
 
@@ -328,9 +337,11 @@ class BasePristineTarSource(UpstreamSource):
         return ret
 
     def has_version(
-            self, package: Optional[str], version: Version, tarballs=None, try_hard=True):
+            self, package: Optional[str], version: str, tarballs=None,
+            try_hard=True):
         if tarballs is None:
-            return self.has_version_component(package, version, component=None, try_hard=try_hard)
+            return self.has_version_component(
+                package, version, component=None, try_hard=try_hard)
         else:
             for (tarball, component, md5) in tarballs:
                 if not self.has_version_component(
@@ -348,8 +359,9 @@ class BasePristineTarSource(UpstreamSource):
         return ret
 
     def version_component_as_revision(
-            self, package: Optional[str], version: Version,
-            component: Optional[str], md5: Optional[str] = None):
+            self, package: Optional[str], version: str,
+            component: Optional[str],
+            md5: Optional[str] = None) -> tuple[RevisionID, str]:
         with self.branch.lock_read():
             for tag_name in self.possible_tag_names(
                     package, version, component=component):
@@ -359,14 +371,14 @@ class BasePristineTarSource(UpstreamSource):
                     continue
                 else:
                     if self._has_revision(revid, md5=md5):
-                        return revid
+                        return revid, ""
             # Note that we don't check *all* possible revids here,
             # since some of them are branch-local (such as revno:)
             (git_id, git_date) = git_snapshot_data_from_version(version)
             if git_id:
                 try:
                     revspec = RevisionSpec.from_string('git:%s' % git_id)
-                    return revspec.as_revision_id(self.branch)
+                    return revspec.as_revision_id(self.branch), ""
                 except (InvalidRevisionSpec, NoSuchTag):
                     pass
             revid = self._search_for_upstream_version(
@@ -377,18 +389,18 @@ class BasePristineTarSource(UpstreamSource):
                     "Upstream import of %s lacks a tag. Set one by running: "
                     "brz tag -rrevid:%s %s", version, revid.decode('utf-8'),
                     tag_name)
-                return revid
+                return revid, ""
             try:
-                return self.branch.tags.lookup_tag(tag_name)
-            except NoSuchTag:
-                raise PackageVersionNotPresent(package, version, self)
+                return self.branch.tags.lookup_tag(tag_name), ""
+            except NoSuchTag as e:
+                raise PackageVersionNotPresent(package, version, self) from e
 
     def _search_for_upstream_version(
             self, package, version, component, md5=None):
         raise NotImplementedError(self._search_for_upstream_version)
 
     def has_version_component(
-            self, package: Optional[str], version: Version, component,
+            self, package: Optional[str], version: str, component,
             md5=None, try_hard=True):
         for tag_name in self.possible_tag_names(
                 package, version, component=component, try_hard=try_hard):
@@ -409,7 +421,7 @@ class BzrPristineTarSource(BasePristineTarSource):
         self.branch = branch
 
     def __repr__(self):
-        return "<%s at %s>" % (self.__class__.__name__, self.branch.base)
+        return "<{} at {}>".format(self.__class__.__name__, self.branch.base)
 
     def tag_name(self, version, component=None, distro=None):
         """Gets the tag name for the upstream part of version.
@@ -423,7 +435,7 @@ class BzrPristineTarSource(BasePristineTarSource):
         if distro is None:
             name = "upstream-" + version
         else:
-            name = "upstream-%s-%s" % (distro, version)
+            name = "upstream-{}-{}".format(distro, version)
         if component is not None:
             name += "/%s" % component
         return name
@@ -463,7 +475,7 @@ class BzrPristineTarSource(BasePristineTarSource):
             if files_excluded_re and files_excluded_re.match(path):
                 return False
             return True
-        message = "Import upstream version %s" % (version,)
+        message = "Import upstream version {}".format(version)
         revprops = {}
         supports_custom_revprops = (
             tree.branch.repository._format.supports_custom_revision_properties)
@@ -479,11 +491,11 @@ class BzrPristineTarSource(BasePristineTarSource):
             if supports_custom_revprops:
                 uuencoded = standard_b64encode(delta).decode('ascii')
                 if tarball.endswith(".tar.bz2"):
-                    revprops[u"deb-pristine-delta-bz2"] = uuencoded
+                    revprops["deb-pristine-delta-bz2"] = uuencoded
                 elif tarball.endswith(".tar.xz"):
-                    revprops[u"deb-pristine-delta-xz"] = uuencoded
+                    revprops["deb-pristine-delta-xz"] = uuencoded
                 else:
-                    revprops[u"deb-pristine-delta"] = uuencoded
+                    revprops["deb-pristine-delta"] = uuencoded
             else:
                 warning('Not setting pristine tar revision properties '
                         'since the repository does not support it.')
@@ -498,12 +510,16 @@ class BzrPristineTarSource(BasePristineTarSource):
             timestamp = timestamp[0]
         if len(parent_ids) == 0:
             base_revid = _mod_revision.NULL_REVISION
+            base_subpath = ""
         else:
-            base_revid = parent_ids[0]
+            base_revid, base_subpath = parent_ids[0]
+        if base_subpath:
+            raise Exception('subpaths are not yet supported')
         basis_tree = tree.branch.repository.revision_tree(base_revid)
         with tree.lock_write():
             builder = tree.branch.get_commit_builder(
-                    parents=parent_ids, revprops=revprops, timestamp=timestamp,
+                    parents=[revid for revid, subpath in parent_ids],
+                    revprops=revprops, timestamp=timestamp,
                     timezone=timezone, committer=committer)
             try:
                 changes = [c for c in tree.iter_changes(basis_tree) if
@@ -523,11 +539,13 @@ class BzrPristineTarSource(BasePristineTarSource):
         return tag_name, revid, delta is not None
 
     def fetch_component_tarball(self, package, version, component, target_dir):
-        revid = self.version_component_as_revision(package, version, component)
+        revid, subpath = self.version_component_as_revision(
+            package, version, component)
+        assert subpath == ""
         try:
             rev = self.branch.repository.get_revision(revid)
-        except NoSuchRevision:
-            raise PackageVersionNotPresent(package, version, self)
+        except NoSuchRevision as e:
+            raise PackageVersionNotPresent(package, version, self) from e
         if revision_has_pristine_tar_delta(rev):
             format = revision_pristine_tar_format(rev)
         else:
@@ -542,14 +560,15 @@ class BzrPristineTarSource(BasePristineTarSource):
         except PristineTarError as e:
             warning('Unable to reconstruct %s using pristine tar: %s',
                     target_filename, e)
-            raise PackageVersionNotPresent(package, version, self)
+            raise PackageVersionNotPresent(package, version, self) from e
         return target_filename
 
     def possible_tag_names(
-            self, package: Optional[str], version: Version,
+            self, package: Optional[str], version: str,
             component: Optional[str],
             try_hard: bool = True):
-        return possible_upstream_tag_names(package, version, component, try_hard=try_hard)
+        return possible_upstream_tag_names(
+            package, version, component, try_hard=try_hard)
 
     def get_pristine_tar_delta(self, package, version, dest_filename,
                                revid=None):
@@ -602,7 +621,7 @@ class BzrPristineTarSource(BasePristineTarSource):
             for component in components]
 
     def _search_for_upstream_version(
-            self, package, version, component, md5=None):
+            self, package: str, version: str, component, md5=None):
         start_revids = []
         sources = []
         sources.append('main branch')
@@ -626,7 +645,7 @@ def get_pristine_tar_source(packaging_tree, packaging_branch):
     return BzrPristineTarSource(packaging_branch)
 
 
-class PristineTarDelta(object):
+class PristineTarDelta:
 
     def __init__(self, tar):
         self._tar = tar
@@ -673,7 +692,7 @@ class GitPristineTarSource(BasePristineTarSource):
         self.packaging_branch = packaging_branch
 
     def __repr__(self):
-        return "<%s at %s>" % (self.__class__.__name__, self.branch.base)
+        return "<{} at {}>".format(self.__class__.__name__, self.branch.base)
 
     @classmethod
     def from_tree(cls, tree, packaging_branch=None):
@@ -721,19 +740,20 @@ class GitPristineTarSource(BasePristineTarSource):
     def tag_name(self, version, component=None, distro=None):
         """Gets the tag name for the upstream part of version.
 
-        :param version: the Version object to extract the upstream
-            part of the version number from.
+        :param version: the upstream vrsion to use
         :param component: Name of the component (None for base)
         :param distro: Optional distribution name
         :return: a String with the name of the tag.
         """
         if self.gbp_tag_format is not None:
-            return gbp_expand_tag_name(self.gbp_tag_format, mangle_version_for_git(version))
+            return gbp_expand_tag_name(
+                self.gbp_tag_format, mangle_version_for_git(version))
         # In git, the convention is to use a slash
         if distro is None:
             name = "upstream/" + mangle_version_for_git(version)
         else:
-            name = "upstream-%s/%s" % (distro, mangle_version_for_git(version))
+            name = "upstream-{}/{}".format(
+                distro, mangle_version_for_git(version))
         if component is not None:
             name += "/%s" % component
         return name
@@ -773,7 +793,7 @@ class GitPristineTarSource(BasePristineTarSource):
             if files_excluded_re and files_excluded_re.match(path):
                 return False
             return True
-        message = "Import upstream version %s" % (version,)
+        message = "Import upstream version {}".format(version)
         revprops = {}
         if component is not None:
             message += ", component %s" % component
@@ -785,12 +805,16 @@ class GitPristineTarSource(BasePristineTarSource):
             timestamp = timestamp[0]
         if len(parent_ids) == 0:
             base_revid = _mod_revision.NULL_REVISION
+            base_subpath = ""
         else:
-            base_revid = parent_ids[0]
+            base_revid, base_subpath = parent_ids[0]
+        if base_subpath:
+            raise Exception('subpaths not yet supported')
         basis_tree = tree.branch.repository.revision_tree(base_revid)
         with tree.lock_write():
             builder = tree.branch.get_commit_builder(
-                    parents=parent_ids, revprops=revprops, timestamp=timestamp,
+                    parents=[revid for revid, parent in parent_ids],
+                    revprops=revprops, timestamp=timestamp,
                     timezone=timezone, committer=committer)
             try:
                 changes = [c for c in tree.iter_changes(basis_tree) if
@@ -819,7 +843,7 @@ class GitPristineTarSource(BasePristineTarSource):
                 builder.finish_inventory()
                 builder.commit('Merge %s.' % version)
 
-        tree_id = revtree._lookup_path(u'')[2]
+        tree_id = revtree._lookup_path('')[2]
         try:
             pristine_tar_branch = self.branch.controldir.open_branch(
                 name='pristine-tar')
@@ -864,7 +888,7 @@ class GitPristineTarSource(BasePristineTarSource):
         except PristineTarError as e:
             warning('Unable to reconstruct %s/%s using pristine tar: %s',
                     package, version, e)
-            raise PackageVersionNotPresent(package, version, self)
+            raise PackageVersionNotPresent(package, version, self) from e
         return target_filename
 
     def _has_revision(self, revid, md5=None):
@@ -882,12 +906,15 @@ class GitPristineTarSource(BasePristineTarSource):
                 return False
         return True
 
-    def possible_tag_names(self, package: Optional[str], version: Version, component: Optional[str], try_hard: bool = True):
+    def possible_tag_names(self, package: Optional[str], version: str,
+                           component: Optional[str], try_hard: bool = True):
         tags = []
         if self.gbp_tag_format:
-            tags.append(gbp_expand_tag_name(self.gbp_tag_format, mangle_version_for_git(version)))
-
-        tags.extend(possible_upstream_tag_names(package, version, component, try_hard=try_hard))
+            tags.append(
+                gbp_expand_tag_name(
+                    self.gbp_tag_format, mangle_version_for_git(version)))
+        tags.extend(possible_upstream_tag_names(
+            package, version, component, try_hard=try_hard))
 
         return tags
 
@@ -901,7 +928,7 @@ class GitPristineTarSource(BasePristineTarSource):
             revtree = pristine_tar_branch.repository.revision_tree(
                 pristine_tar_branch.last_revision())
             for suffix in self.SUFFIXES:
-                basename = '%s_%s.orig.%s' % (package, version, suffix)
+                basename = '{}_{}.orig.{}'.format(package, version, suffix)
                 try:
                     delta_bytes = revtree.get_file_text(basename + '.delta')
                 except NoSuchFile:
@@ -918,24 +945,26 @@ class GitPristineTarSource(BasePristineTarSource):
             self, package, version, component, target_dir):
         """Reconstruct a pristine-tar tarball from a git revision."""
         try:
-            dest_filename, delta_bytes, delta_id, delta_sig = self.get_pristine_tar_delta(
-                package, version)
+            (dest_filename, delta_bytes, delta_id,
+             delta_sig) = self.get_pristine_tar_delta(package, version)
         except PristineTarDeltaAbsent:
-            revid = self.version_component_as_revision(
+            revid, subpath = self.version_component_as_revision(
                 package, version, component)
             tree = self.branch.repository.revision_tree(revid)
             dest_filename = self._tarball_path(
                 package, version, component, target_dir, format='gz')
-            export_with_nested(tree, dest_filename, per_file_timestamps=True)
+            export_with_nested(
+                tree, dest_filename, per_file_timestamps=True, subdir=subpath)
             return dest_filename
         else:
             dest_filename = os.path.join(target_dir, dest_filename)
             try:
                 subprocess.check_call(
                     ['pristine-tar', 'checkout', dest_filename],
-                    cwd=self.branch.repository.user_transport.local_abspath('.'))
+                    cwd=self.branch.repository.user_transport.local_abspath(
+                        '.'))
             except subprocess.CalledProcessError as e:
-                raise PristineTarError(str(e))
+                raise PristineTarError(str(e)) from e
             return dest_filename
 
     def _components_by_pristine_tar(self, package=None):
@@ -984,10 +1013,13 @@ class GitPristineTarSource(BasePristineTarSource):
         if components is None:
             # Scan tags for components
             try:
-                components = self._components_by_version()[version].keys()
+                components = list(
+                    self._components_by_version()[version].keys())
             except KeyError:
                 try:
-                    components = self._components_by_pristine_tar(package)[version].keys()
+                    components = list(
+                        self._components_by_pristine_tar(
+                            package)[version].keys())
                 except KeyError:
                     components = [None]
         return [
@@ -996,7 +1028,7 @@ class GitPristineTarSource(BasePristineTarSource):
             for component in components]
 
     def _search_for_upstream_version(
-            self, package: Optional[str], version: Version, component,
+            self, package: Optional[str], version: str, component,
             md5=None):
         start_revids = []
         sources = []
